# Serene Bach plugin for fold category.
#
# Release 1.00 (December  9,2005)
#         1.01 (February 12,2006) corrected trouble to which the number of categories was not counted.
#         2.00 (February xx,2007) add function for sum subcategory entry number.
#         2.01 (July      3,2008) fix bug.
#
# This software is provided as-is. You may use it for commercial or 
# personal use. If you distribute it, please keep this notice intact.
#
# Copyright (c) 2005-2006 Yujiro Araki

package sb::Plugin::FoldCategory;

use sb::Plugin ();
use vars qw($js,@history);

sb::Plugin->register_plugin(
    'lang' => {
        'ja' => 'euc',
        'en' => 'ascii',
    },
    'text' => {
        'type'    => 'cms',
        'name'    => 'FoldCategory',
        'text'    => 'Fold Category List',
        'author'  => 'Yujiro Araki',
        'detail'  => 'http://www.koikikukan.com/',
        'version' => '1.00',
    },
    'data' => 1,
    'file' => 'foldcategory.txt',
);

# register callback function to cms entry point
sb::Plugin->register_content_module(
    'type'     => 'main',
    'callback' => \&sb::Content::FoldCategory::_fold_category_list,
    'field'    => 'fold_category',
);


# register administration module
sb::Plugin->register_admin_module(
	'mode'   => 'foldcategory',
	'level'  => 1, # for advanced users and the administrator
	'module' => 'sb::Admin::FoldCategory',
);

package sb::Content::FoldCategory;


# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::TemplateManager ();

our $parenthesis_l;
our $parenthesis_r;
our $disp_number_flag;
our $parent_category;

sub _get_option {
	my $data = sb::Plugin->get_data;
	my @format = ($data->text) ? split(/<>/, $data->text) : sb::Admin::FoldCategory->mCatDefault();
    $parenthesis_l = $format[0];
    $parenthesis_r = $format[1];
    $disp_number_flag = $data->setting;
    $parent_category = $data->date;
}


sub _fold_category_list { # JeS[Xg
    my $cms = shift;
    my %var = @_;

    if ($var{'mode'} eq 'ent') {
        $var{'entryinfo'} = &_entry_info(%var);
    }

    &_get_option;

    my ($all_list,$sub_list);
    my $cat = undef;
    $js = '';
    $all_list = &_category_tree(
        'cat' => $var{'sortedcat'},
        'num' => $var{'entryinfo'}{'category'},
    );

    # ppf[^Xg폜
    $all_list =~ s/sum:\d+//;

    $cat = $var{'cat'}->{$var{'id'}} if ($var{'mode'} eq 'cat');
    if ($var{'mode'} eq 'ent' and $var{'entry'}->[0]) {
        my $cat_id = $var{'entry'}->[0]->cat;
        $cat = $var{'cat'}->{$cat_id} if ($cat_id ne '');
    }
    if ($cat) {
        $sub_list = &_category_tree(
            'cat'    => $var{'sortedcat'},
            'branch' => $cat->id,
            'num'    => $var{'entryinfo'}{'category'},
        );
    }
    $js = "\n".'<script type="text/javascript">'."\n<!--\n".$js."//-->\n</script>\n";
    $cms->num(0);
    $cms->tag('fold_category_list' => ($all_list) ? '<ul>' . $all_list . '</ul>' . $js : '');
    $cms->tag('fold_category_script' => '<script type="text/javascript" src="'.$var{'conf'}->value('conf_srv_base').'menufolder.js" charset="utf-8"></script>');
    return 1;
}

sub _category_tree { # JeS[c[
    my %param = (
        'cat'    => [],
        'branch' => undef,
        'num'    => {},
        'no_num' => undef,
        'counter' => undef,
        @_
    );
    my $list = '';
    my $same;
    if(!defined($param{'counter'})){
        $param{'counter'} = 0;
        push(@history,$param{'counter'});
    }
    foreach $i (@history){
        if($param{'counter'} == $i) {
            $same = 1;
        }
    }
    if($same){
        $param{'counter'}*=100;
    }

    my $sum;
    foreach my $cat ( @{$param{'cat'}} ) {
        next if ($cat->get_option('list') == 1);
        next if (!defined($param{'branch'}) and $cat->main ne '');
        next if ( defined($param{'branch'}) and $cat->main ne $param{'branch'});

        # TuJeS[ێp
        my $child_list = "";

        my $text = $cat->text; # code by Fuco. Thanks!
        $text = ($text ne '') ? ' title="' . sb::Text->entitize($text) . '"' : '';
        if ($cat->sub ne '') {
            $list .= '<li><div class="subcategories" id="subcategories' . $param{'counter'} . 'name"><a href="' . $cat->cat_url . '"' . $text . '>' . $cat->name . '</a>';
        } else {
            $list .= '<li><a href="' . $cat->cat_url . '"' . $text . '>' . $cat->name . '</a>';
        }

        # eJeS[ɃTuJeS[܂܂Ȃꍇ̃JeS[JEg
        if($disp_number_flag){
            if(!$parent_category){
#                $list .= $parenthesis_l . int($param{'num'}->{$cat->id}) . $parenthesis_r if (!$param{'no_num'} && $param{'num'}->{$cat->id});
                $list .= $parenthesis_l . int($param{'num'}->{$cat->id}) . $parenthesis_r if (!$param{'no_num'});
            }
        }
        if ($cat->sub ne '') {
            $child_list .= '</div>';
        }

        # JeS[̋L\OɃTuJeS[Xg擾
        if ($cat->sub ne '') {
            $child_list .= "\n" . '<div id="subcategories' . $param{'counter'} . 'list"><ul>';
            $js .= "FoldNavigation('subcategories" . $param{'counter'} . "','on',false);\n";
            $param{'counter'}++;
            push(@history,$param{'counter'});
            $child_list .= &_category_tree(
                'cat'     => $param{'cat'},
                'branch'  => $cat->id,
                'num'     => $param{'num'},
                'no_num'  => $param{'no_num'},
                'counter' => $param{'counter'},
            );
            $child_list .= '</ul></div>';
        }

        # TuJeS[L𒊏o($child_listɊ܂܂'sum:n')
        my $child_num = 0;
        if($child_list =~ /sum:(\d+)/) {
            $child_num = $1;
        }
        $child_list =~ s/sum:\d+//;

        # eJeS[LɃTuJeS[LZAXg֒ǉ
        my $current_sum = int($param{'num'}->{$cat->id}) + $child_num;

        # eJeS[ɃTuJeS[܂ޏꍇ̃JeS[JEg
        if($disp_number_flag){
            if($parent_category){
                $list .= $parenthesis_l . $current_sum . $parenthesis_r if ($current_sum);
            }
        }
#        $list .= ' (' . $current_sum . ')' if ($current_sum);

        # TuJeS[݂ꍇ̓TuJeS[ŜXgɒǉ
        $list .= $child_list if ($cat->sub ne '');

        # ݂̍vLeJeS[ppɕێ
        $sum += $current_sum;

        $list .= '</li>'  . "\n";
    }
    return($list . 'sum:' . $sum);
}

sub _entry_info { # eGg[̎擾
    my %var = @_;
    my %info = (
        'latest'   => [],
        'monthly'  => {},
        'daily'    => {},
        'neighbor' => {'prev'=>undef,'next'=>undef,},
        'category' => {},
    );
    my $check = -1;
    my @array = sb::Data->load('Entry','cond'=>{'stat'=>[1,2]},'sort'=>'date','order'=>1);
    for (my $i=0;$i<@array;$i++) {
        my $entry = $array[$i];
        $info{'category'}{$entry->cat}++ if ($entry->cat ne ''); # JeS[̃JEg
        if ($entry->add ne '') { # ֘AJeS[̃JEg
            foreach ( split(',',$entry->add) ) {
                next if ($_ eq '');
                $info{'category'}{$_}++;
            }
        }
        if ( ($var{'mode'} eq 'ent' and $var{'id'} == $entry->id)
          or ($var{'mode'} eq 'mob' and $var{'id'} ne '' and $var{'id'} == $entry->id) ) {
            $check = $i; # ÕGg[i[
        }
    }
    $info{'neighbor'}{'next'} = $array[$check - 1] if ($check > 0);
    $info{'neighbor'}{'prev'} = $array[$check + 1] if ($check > -1 and $check < $#array);
    return(\%info);
}


#------------------------------------------------------------------------------------------

package sb::Admin::FoldCategory;
# ==================================================
# // declaration for global variables
# ==================================================
use vars qw( @ISA );
# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Interface ();
use sb::TemplateManager ();
use sb::Text ();
@ISA = qw( sb::App::Admin );
# ==================================================
# // declaration for constant value
# ==================================================
sub TEMPLATE      (){ 'foldcategory.html' };
sub ATTR_CHECKED  (){ 'checked="checked"' };
sub ATTR_SELECTED (){ 'selected="selected"' };
# ==================================================
# // declaration for data structure
# ==================================================
my @mCatQuery = (
#    'disp_num',
#    'add_child',
    'left_char',
    'right_char', 
);
my @mCatDefault = (
#    'checked',
#    '',
    '[',   # left
    ']',   # right
);

# ==================================================
# // public functions - callback
# ==================================================
sub callback { # callbacks
    my $self = shift;
    return ( $self->{'regi'} ) 
        ? $self->_save_setting(@_)
        : $self->_open_ui(@_);
}

sub get_tree_default {
    return @mTreeDefault;
}

sub get_prefix_default {
    return @mCatDefault;
}

# ==================================================
# // private functions
# ==================================================
sub _save_setting {
    my $self = shift;
    my $cgi  = sb::Interface->get;
    return $self->_save_basic_setting($cgi, @_) ;
}

sub _open_ui {
    my $self = shift;
    my $cgi  = sb::Interface->get;
    return $self->_open_basic_menu(@_) ;
}

sub _save_basic_setting {
    my $self = shift;
    my $cgi  = shift;
    my $data = sb::Plugin->get_data;
    my $lang = sb::Language->get;
    my %errmsg = (
        'message'=>'',
        'trackback'=>'',
    );

    $data->setting(($cgi->value('disp_num') eq 'on') ? 1 : 0);
    my @values = ($data->text) ? split(/<>/,$data->text) : @mCatDefault;

    my $num = 0;
    foreach (@mCatQuery) {
        my $value = $cgi->value($_);
        if ($value eq '') {
            $values[$num] = $mCatDefault[$num];
        } else {
#            if ($value =~ /^[A-z][\w\-:]*$/) {
                $values[$num] = $value;
#            } else {
#                $errmsg{$_} = $value . ': ' . sb::Language->get->string('error_wrong_text');
#            }
        }
        $num++;
    }
    $data->text(join("<>", @values));
    $data->date($cgi->value('add_child'));

    sb::Plugin->set_data('data'=>$data);

    my $msg = $lang->string('parts_confcomp');
    $msg .= $lang->string('parts_rec_make');
    $msg .= sprintf($lang->string('parts_link_bld'),$self->get_script_path);

    return $self->_open_basic_menu('done'=>$msg, %errmsg);
}

sub _open_basic_menu {
    my $self = shift;
    my %param = ('message' => '', @_);
    my $cms  = sb::TemplateManager->new(
                    sb::Plugin->load_template('file'=>TEMPLATE));
    my $data = sb::Plugin->get_data;

    $cms->block('sb_foldcategory_menu_basic'=>1);
    $cms->num(0);
    $cms->tag('sb_foldcategory_tab_basic'=>'class="current"');
    $cms->tag('sb_foldcategory_disp_num'=>ATTR_CHECKED) if ($data->setting);

    if (!$data->date) {
        $cms->tag('sb_foldcategory_no_child'=>ATTR_SELECTED);
    } else {
        $cms->tag('sb_foldcategory_add_child'=>ATTR_SELECTED);
    }

    my @values = ($data->text ne '') ? split(/<>/, $data->text) : @mCatDefault;
    foreach my $key (@mCatQuery) {
        my $value = shift @values;
        $cms->tag('sb_foldcategory_' . $key =>sb::Text->entitize($value));
        last if ($#values < 0);
    }

    # display massage if requested
    if ($param{'done'} ne '') {
        $cms->num(0);
        $cms->tag('sb_process_message'=>$param{'done'});
        $cms->block('sb_process_message'=>1);

        my $errnum = 0;
        foreach (@mCatQuery) {
            if ($param{$_} ne '') {
                $cms->num($errnum);
                $cms->tag('sb_process_err_msg'=>$param{$_});
                $errnum++;
            }
        }
        $cms->block('sb_process_err_msg'=>1) if ($errnum > 0);
    }

    # display administration user interface
    return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}


1;
__END__
